#ifndef STA_RASPI_I2C_HPP
#define STA_RASPI_I2C_HPP

#include <sta/config.hpp>
#ifdef STA_PLATFORM_RASPI

#include <sta/bus/i2c/i2c.hpp>
#include <sta/bus/i2c/device.hpp>

#include <linux/i2c-dev.h>
#include <sys/ioctl.h>

namespace sta 
{
    enum class I2cNode {
        DEV_1,
        DEV_2
    };

    class RaspiI2c : public I2c 
    {
    public:
        RaspiI2c(I2cNode node, Mutex * mutex=nullptr, bool persistent_open=false);
        ~RaspiI2c();

        void transfer(uint8_t value) override;
        void transfer16(uint16_t value) override;
        void transfer(const uint8_t * buffer, size_t size) override;
        void transfer(const uint8_t * txBuffer, uint8_t * rxBuffer, size_t size) override;
        void receive(uint8_t * buffer, size_t size) override;

        void fill(uint8_t value, size_t count) override;

        void selectAddress(uint16_t address) override; 
        void acquire() override;
        void release() override;
    private:
        char * i2cdev_;
        int i2cfd_;
        bool open_ = false;
        const bool persistent_open_;
    };

    class RaspiI2cDevice : public I2cDevice
    {
        RaspiI2cDevice(I2c * intf, uint16_t address_10bit, Mutex* mutex=nullptr, bool master=false, bool blocking=true);
    };
} // namespace sta

#endif // STA_PLATFORM_RASPI

#endif // STA_I2C_HPP